{% if form.non_field_errors %}
    <div class="notification is-danger">
        <button class="delete"></button>

        {% if form_error_title %}<div class="has-text-weight-bold">{{ form_error_title }}</div>{% endif %}
        <ul>
            {{ form.non_field_errors|unordered_list }}
        </ul>
    </div>
{% endif %}